from pathlib import Path
from sys import argv, exit
from typing import Dict, Optional
import importlib
import sys

sys.path.append("./")

from pkg_resources import get_distribution

from .internal.taskcontext import TaskContext
from .internal.taskfunc import TaskFunc

dist = get_distribution("please_av1ppp")

pleasefile_module = "Pleasefile"
pleasefile_name = Path(pleasefile_module + ".py")


init_command = ["-i", "-init"]
help_command = ["-h", "-help"]
version_command = ["-v", "-version"]

help_indent = "    "


def main():
    tasks = get_tasks_from_pleasefile()
    args = argv[1:]

    if len(args) == 0 or args[0] in help_command:
        print_help(tasks)
        return

    if args[0] in init_command:
        init_pleasefile()
        return

    if args[0] in version_command:
        print_version()
        return

    if tasks is not None and args[0] in tasks:
        task = tasks[args[0]]
        ctx = TaskContext(args[1:])
        task(ctx)
        return

    panic(f"Command or task '{args[0]}' not found. Try to use -h command.")


def init_pleasefile():
    if pleasefile_name.exists():
        panic("Pleasefile already created")

    with open(pleasefile_name, mode="w") as file:
        file.write(
            """import please


@please.task()
def start(ctx: please.TaskContext):
    mode = ctx.string("mode") or "prod"
    print(f"*starting app in {mode} mode*")
"""
        )


def print_version():
    print("Please v" + dist.version)


def print_help(tasks: Optional[Dict[str, TaskFunc]]):
    print("PLEASE - simple task runner.")
    print()

    print("COMMANDS:")
    print_command(init_command, "Create empty Pleasefile")
    print_command(help_command, "Show this message")
    print_command(version_command, "Show version")

    if tasks is not None and len(tasks) > 0:
        print()
        print("TASKS:")
        for task_name, _ in tasks.items():
            print(f"{help_indent}{task_name}")


def get_tasks_from_pleasefile():
    try:
        module = importlib.import_module(pleasefile_module)
    except ModuleNotFoundError:
        return None

    try:
        tasks: Dict[str, TaskFunc] = module.please.internal.tasks.tasks
    except AttributeError:
        return None

    return tasks


def print_command(command: list, description: str):
    print(help_indent + ", ".join(command).ljust(26, " ") + description)


def panic(*values: object):
    print("ERROR:", *values)
    exit(1)


if __name__ == "__main__":
    main()



from typing import List, Optional

from .argument import parse_args

_true_values = ["", "on", "yes", "y", "true"]
_false_values = ["off", "no", "n", "false"]


class TaskContext:
    def __init__(self, args: List[str]) -> None:
        self._args = parse_args(args)

    def args(self) -> List[str]:
        return list(
            map(
                lambda arg: arg.value,
                filter(
                    lambda arg: arg.key is None,
                    self._args,
                ),
            )
        )

    def string(self, key: str) -> Optional[str]:
        return self._find_by_key(key)

    def muststring(self, key: str):
        value = self.string(key)
        if value is None:
            raise Exception(f"argument '{key}' must be specified")
        return value

    def boolean(self, key: str) -> Optional[bool]:
        value = self._find_by_key(key)
        if value is None:
            return None
        if value in _true_values:
            return True
        elif value in _false_values:
            return False
        raise Exception(f"string '{value}' cannot be interpreted as a boolean value")

    def mustboolean(self, key: str):
        value = self.boolean(key)
        if value is None:
            raise Exception(f"argument '{key}' must be specified")
        return value

    def integer(self, key: str) -> Optional[int]:
        value = self._find_by_key(key)
        if value is None:
            return None
        try:
            value_ = int(value)
        except ValueError:
            raise Exception(f"string '{value}' cannot be interpreted as a int value")
        return value_

    def mustinteger(self, key: str):
        value = self.integer(key)
        if value is None:
            raise Exception(f"argument '{key}' must be specified")
        return value

    def float(self, key: str) -> Optional[float]:
        value = self._find_by_key(key)
        if value is None:
            return None
        try:
            value_ = float(value)
        except ValueError:
            raise Exception(f"string '{value}' cannot be interpreted as a float value")
        return value_

    def mustfloat(self, key: str):
        value = self.float(key)
        if value is None:
            raise Exception(f"argument '{key}' must be specified")
        return value

    def _find_by_key(self, key: str) -> Optional[str]:
        for arg in self._args:
            if arg.key == key:
                return arg.value
        return None



from dataclasses import dataclass
from typing import List, Optional


@dataclass()
class Argument:
    key: Optional[str]
    value: str


def parse_args(args: List[str]) -> List[Argument]:
    result: List[Argument] = []

    for arg in args:
        if arg.startswith("-"):
            parts = arg[1:].split("=", 2)
            if len(parts) == 1:
                value = ""
            else:
                value = parts[1]
            arg_ = Argument(key=parts[0], value=value)
        else:
            arg_ = Argument(key=None, value=arg)

        result.append(arg_)

    return result



from os import environ
from pathlib import Path
from typing import Union

PathLike = Union[str, Path]


def parseenv(path: PathLike):
    env: dict[str, str] = dict()

    with open(path, mode="r") as file:
        for line in file.readlines():
            line = line.strip()
            if len(line) == 0 or line.startswith("#"):
                continue

            splitted = line.split("=", maxsplit=2)
            env[splitted[0].strip()] = splitted[1].strip()

    return env


def loadenv(path: PathLike):
    env = parseenv(path)
    for key, value in env.items():
        environ[key] = value
